@@ -16,11 +16,7 @@
 
 #define IP6T_HL_USED	1
 
-static void init(struct ip6t_entry_target *t, unsigned int *nfcache) 
-{
-}
-
-static void help(void) 
+static void HL_help(void)
 {
 	printf(
 "HL target v%s options\n"
@@ -30,9 +26,8 @@ static void help(void)
 , IPTABLES_VERSION);
 }
 
-static int parse(int c, char **argv, int invert, unsigned int *flags,
-		const struct ip6t_entry *entry,
-		struct ip6t_entry_target **target)
+static int HL_parse(int c, char **argv, int invert, unsigned int *flags,
+                    const void *entry, struct xt_entry_target **target)
 {
 	struct ip6t_HL_info *info = (struct ip6t_HL_info *) (*target)->data;
 	unsigned int value;
@@ -89,15 +84,14 @@ static int parse(int c, char **argv, int invert, unsigned int *flags,
 	return 1;
 }
 
-static void final_check(unsigned int flags)
+static void HL_check(unsigned int flags)
 {
 	if (!(flags & IP6T_HL_USED))
 		exit_error(PARAMETER_PROBLEM,
 				"HL: You must specify an action");
 }
 
-static void save(const struct ip6t_ip6 *ip,
-		const struct ip6t_entry_target *target)
+static void HL_save(const void *ip, const struct xt_entry_target *target)
 {
 	const struct ip6t_HL_info *info = 
 		(struct ip6t_HL_info *) target->data;
@@ -117,8 +111,8 @@ static void save(const struct ip6t_ip6 *ip,
 	printf("%u ", info->hop_limit);
 }
 
-static void print(const struct ip6t_ip6 *ip,
-		const struct ip6t_entry_target *target, int numeric)
+static void HL_print(const void *ip, const struct xt_entry_target *target,
+                     int numeric)
 {
 	const struct ip6t_HL_info *info =
 		(struct ip6t_HL_info *) target->data;
@@ -138,29 +132,27 @@ static void print(const struct ip6t_ip6 *ip,
 	printf("%u ", info->hop_limit);
 }
 
-static struct option opts[] = {
-	{ "hl-set", 1, 0, '1' },
-	{ "hl-dec", 1, 0, '2' },
-	{ "hl-inc", 1, 0, '3' },
-	{ 0 }
+static const struct option HL_opts[] = {
+	{ "hl-set", 1, NULL, '1' },
+	{ "hl-dec", 1, NULL, '2' },
+	{ "hl-inc", 1, NULL, '3' },
+	{ }
 };
 
-static
-struct ip6tables_target HL = { NULL, 
+static struct ip6tables_target hl_target6 = {
 	.name 		= "HL",
 	.version	= IPTABLES_VERSION,
 	.size		= IP6T_ALIGN(sizeof(struct ip6t_HL_info)),
 	.userspacesize	= IP6T_ALIGN(sizeof(struct ip6t_HL_info)),
-	.help		= &help, 
-	.init		= &init,
-	.parse		= &parse,
-	.final_check	= &final_check,
-	.print		= &print,
-	.save		= &save,
-	.extra_opts	= opts 
+	.help		= HL_help,
+	.parse		= HL_parse,
+	.final_check	= HL_check,
+	.print		= HL_print,
+	.save		= HL_save,
+	.extra_opts	= HL_opts,
 };
 
 void _init(void)
 {
-	register_target6(&HL);
+	register_target6(&hl_target6);
 }
