@@ -8,9 +8,7 @@
 #include<linux/netfilter_ipv6/ip6_tables.h>
 #include<linux/netfilter_ipv6/ip6t_condition.h>
 
-
-static void
-help(void)
+static void condition_help(void)
 {
 	printf("condition match v%s options:\n"
 	       "--condition [!] filename       "
@@ -18,16 +16,14 @@ help(void)
 	       IPTABLES_VERSION);
 }
 
-
-static struct option opts[] = {
+static const struct option condition_opts[] = {
 	{ .name = "condition", .has_arg = 1, .flag = 0, .val = 'X' },
 	{ .name = 0 }
 };
 
 static int
-parse(int c, char **argv, int invert, unsigned int *flags,
-      const struct ip6t_entry *entry, unsigned int *nfcache,
-      struct ip6t_entry_match **match)
+condition_parse(int c, char **argv, int invert, unsigned int *flags,
+                const void *entry, struct xt_entry_match **match)
 {
 	struct condition6_info *info =
 	    (struct condition6_info *) (*match)->data;
@@ -53,19 +49,15 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 	return 0;
 }
 
-
-static void
-final_check(unsigned int flags)
+static void condition_check(unsigned int flags)
 {
 	if (!flags)
 		exit_error(PARAMETER_PROBLEM,
 			   "Condition match: must specify --condition");
 }
 
-
-static void
-print(const struct ip6t_ip6 *ip,
-		  const struct ip6t_entry_match *match, int numeric)
+static void condition_print(const void *ip, const struct xt_entry_match *match,
+                            int numeric)
 {
 	const struct condition6_info *info =
 	    (const struct condition6_info *) match->data;
@@ -74,9 +66,7 @@ print(const struct ip6t_ip6 *ip,
 }
 
 
-static void
-save(const struct ip6t_ip6 *ip,
-		 const struct ip6t_entry_match *match)
+static void condition_save(const void *ip, const struct xt_entry_match *match)
 {
 	const struct condition6_info *info =
 	    (const struct condition6_info *) match->data;
@@ -84,23 +74,22 @@ save(const struct ip6t_ip6 *ip,
 	printf("--condition %s\"%s\" ", (info->invert) ? "! " : "", info->name);
 }
 
-
-static struct ip6tables_match condition = {
+static struct ip6tables_match condition_match6 = {
 	.name = "condition",
 	.version = IPTABLES_VERSION,
 	.size = IP6T_ALIGN(sizeof(struct condition6_info)),
 	.userspacesize = IP6T_ALIGN(sizeof(struct condition6_info)),
-	.help = &help,
-	.parse = &parse,
-	.final_check = &final_check,
-	.print = &print,
-	.save = &save,
-	.extra_opts = opts
+	.help = condition_help,
+	.parse = condition_parse,
+	.final_check = condition_check,
+	.print = condition_print,
+	.save = condition_save,
+	.extra_opts = condition_opts,
 };
 
 
 void
 _init(void)
 {
-	register_match6(&condition);
+	register_match6(&condition_match6);
 }
