@@ -18,12 +18,11 @@
 #include <errno.h>
 
 #include <iptables.h>
-#include <linux/netfilter_ipv4/ip_conntrack.h>
 #include <linux/netfilter_ipv4/ipt_set.h>
 #include "libipt_set.h"
 
 /* Function which prints out usage message. */
-static void help(void)
+static void set_help(void)
 {
 	printf("set v%s options:\n"
 	       " [!] --set     name flags\n"
@@ -33,13 +32,13 @@ static void help(void)
 	       "\n", IPTABLES_VERSION);
 }
 
-static struct option opts[] = {
+static const struct option set_opts[] = {
 	{"set", 1, 0, '1'},
 	{0}
 };
 
 /* Initialize the match. */
-static void init(struct ipt_entry_match *match, unsigned int *nfcache)
+static void set_init(struct xt_entry_match *match)
 {
 	struct ipt_set_info_match *info = 
 		(struct ipt_set_info_match *) match->data;
@@ -50,10 +49,8 @@ static void init(struct ipt_entry_match *match, unsigned int *nfcache)
 }
 
 /* Function which parses command options; returns true if it ate an option */
-static int
-parse(int c, char **argv, int invert, unsigned int *flags,
-      const struct ipt_entry *entry,
-      unsigned int *nfcache, struct ipt_entry_match **match)
+static int set_parse(int c, char **argv, int invert, unsigned int *flags,
+                     const void *entry, struct xt_entry_match **match)
 {
 	struct ipt_set_info_match *myinfo = 
 		(struct ipt_set_info_match *) (*match)->data;
@@ -96,7 +93,7 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 }
 
 /* Final check; must have specified --set. */
-static void final_check(unsigned int flags)
+static void set_check(unsigned int flags)
 {
 	if (!flags)
 		exit_error(PARAMETER_PROBLEM,
@@ -112,7 +109,7 @@ print_match(const char *prefix, const struct ipt_set_info *info)
 
 	get_set_byid(setname, info->index);
 	printf("%s%s %s", 
-	       (info->flags[0] & IPSET_MATCH_INV) ? "!" : "",
+	       (info->flags[0] & IPSET_MATCH_INV) ? "! " : "",
 	       prefix,
 	       setname); 
 	for (i = 0; i < IP_SET_MAX_BINDINGS; i++) {
@@ -126,9 +123,8 @@ print_match(const char *prefix, const struct ipt_set_info *info)
 }
 
 /* Prints out the matchinfo. */
-static void
-print(const struct ipt_ip *ip,
-      const struct ipt_entry_match *match, int numeric)
+static void set_print(const void *ip, const struct xt_entry_match *match,
+                      int numeric)
 {
 	struct ipt_set_info_match *info = 
 		(struct ipt_set_info_match *) match->data;
@@ -137,8 +133,7 @@ print(const struct ipt_ip *ip,
 }
 
 /* Saves the matchinfo in parsable form to stdout. */
-static void save(const struct ipt_ip *ip,
-		 const struct ipt_entry_match *match)
+static void set_save(const void *ip, const struct xt_entry_match *match)
 {
 	struct ipt_set_info_match *info = 
 		(struct ipt_set_info_match *) match->data;
@@ -146,22 +141,21 @@ static void save(const struct ipt_ip *ip,
 	print_match("--set", &info->match_set);
 }
 
-static
-struct iptables_match set = {
+static struct iptables_match set_match = {
 	.name		= "set",
 	.version	= IPTABLES_VERSION,
 	.size		= IPT_ALIGN(sizeof(struct ipt_set_info_match)),
 	.userspacesize	= IPT_ALIGN(sizeof(struct ipt_set_info_match)),
-	.help		= &help,
-	.init		= &init,
-	.parse		= &parse,
-	.final_check	= &final_check,
-	.print		= &print,
-	.save		= &save,
-	.extra_opts	= opts
+	.help		= set_help,
+	.init		= set_init,
+	.parse		= set_parse,
+	.final_check	= set_check,
+	.print		= set_print,
+	.save		= set_save,
+	.extra_opts	= set_opts,
 };
 
 void _init(void)
 {
-	register_match(&set);
+	register_match(&set_match);
 }
